var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import CodeSuggestionCache from './CodeSuggestionCache';
import TurndownService from 'turndown';
import { languages } from 'monaco-editor';
var CodeSuggester = /** @class */ (function () {
    function CodeSuggester(xsdParser) {
        var _this = this;
        this.elements = function (parentElement, namespace, withoutTag, incomplete) {
            if (withoutTag === void 0) { withoutTag = false; }
            if (incomplete === void 0) { incomplete = false; }
            return _this.parseElements(_this.codeSuggestionCache.elements(parentElement), namespace, withoutTag, incomplete);
        };
        this.attributes = function (element, incomplete) {
            if (incomplete === void 0) { incomplete = false; }
            return _this.parseAttributes(_this.codeSuggestionCache.attributes(element), incomplete);
        };
        this.parseElements = function (elements, namespace, withoutTag, incomplete) {
            return elements.map(function (element, index) {
                var elementName = element.ref ? element.ref : element.name;
                if (elementName) {
                    elementName = _this.parseElementName(elementName, namespace);
                }
                return {
                    label: elementName,
                    kind: withoutTag
                        ? languages.CompletionItemKind.Snippet
                        : languages.CompletionItemKind.Method,
                    detail: _this.parseDetail(element.type),
                    sortText: index.toString(),
                    insertText: _this.parseElementInputText(elementName, withoutTag, incomplete),
                    insertTextRules: languages.CompletionItemInsertTextRule.InsertAsSnippet,
                    documentation: _this.parseDocumentation(element.documentation),
                };
            });
        };
        this.parseElementName = function (name, namespace) {
            return namespace ? namespace + ':' + name : name;
        };
        this.parseElementInputText = function (name, withoutTag, incomplete) {
            if (withoutTag)
                return '<' + name + '${1}>\n\t${2}\n</' + name + '>';
            if (incomplete)
                return name;
            return name + '${1}></' + name;
        };
        this.parseAttributes = function (attributes, incomplete) {
            return attributes.map(function (attribute) {
                var attributeName = attribute.ref ? attribute.ref : attribute.name;
                return ({
                    label: attributeName,
                    kind: languages.CompletionItemKind.Variable,
                    detail: _this.parseDetail(attribute.type),
                    insertText: _this.parseAttributeInputText(attributeName, incomplete),
                    preselect: _this.attributeIsRequired(attribute),
                    insertTextRules: languages.CompletionItemInsertTextRule.InsertAsSnippet,
                    documentation: _this.parseDocumentation(attribute.documentation),
                });
            });
        };
        this.parseDetail = function (detail) {
            if (detail) {
                var _a = __read(detail.split(':'), 2), partOne = _a[0], partTwo = _a[1];
                return partTwo !== null && partTwo !== void 0 ? partTwo : partOne;
            }
        };
        this.parseDocumentation = function (documentation) { return ({
            value: documentation ? _this.turndownService.turndown(documentation) : '',
            isTrusted: true,
        }); };
        this.attributeIsRequired = function (attribute) { return attribute.use === 'required'; };
        this.parseAttributeInputText = function (name, incomplete) {
            return incomplete ? name : name + '="${1}"';
        };
        this.codeSuggestionCache = new CodeSuggestionCache(xsdParser);
        this.turndownService = new TurndownService();
    }
    return CodeSuggester;
}());
export default CodeSuggester;
//# sourceMappingURL=CodeSuggester.js.map