"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var monaco_editor_1 = require("monaco-editor");
var CompletionTriggerKind = monaco_editor_1.languages.CompletionTriggerKind;
var CompletionItemKind = monaco_editor_1.languages.CompletionItemKind;
var types_1 = require("./types");
var SimpleParser_1 = require("./SimpleParser");
var XsdNamespaces_1 = require("./XsdNamespaces");
var XsdCompletion = /** @class */ (function () {
    function XsdCompletion(xsdCollection) {
        var _this = this;
        this.provider = function () { return ({
            triggerCharacters: ['<', ' ', '/'],
            provideCompletionItems: function (model, position, context) { return ({
                suggestions: _this.getCompletionItems(model, position, context),
            }); },
        }); };
        this.getCompletionItems = function (model, position, context) {
            var completions = _this.getCompletions(model, position, context);
            var wordUntilPosition = model.getWordUntilPosition(position);
            var tagBeforePosition = _this.getLastTagBeforePosition(model, position);
            var startColumn = _this.getStartColumnForTagWithNamespace(wordUntilPosition, tagBeforePosition);
            var wordRange = {
                startColumn: startColumn,
                startLineNumber: position.lineNumber,
                endColumn: wordUntilPosition.endColumn,
                endLineNumber: position.lineNumber,
            };
            return completions.map(function (completion) {
                return __assign(__assign({}, completion), { range: wordRange });
            });
        };
        this.getStartColumnForTagWithNamespace = function (wordUntilPosition, tagBeforePosition) {
            if (wordUntilPosition &&
                tagBeforePosition &&
                wordUntilPosition.word === _this.getTagName(tagBeforePosition)) {
                var lengthDifferance = Math.abs(tagBeforePosition.length - wordUntilPosition.word.length);
                return wordUntilPosition.startColumn - lengthDifferance;
            }
            return wordUntilPosition.startColumn;
        };
        this.getCompletions = function (model, position, context) {
            var completionType = _this.getCompletionType(model, position, context);
            if (completionType == types_1.CompletionType.none)
                return [];
            var parentTag = _this.getParentTag(model, position);
            if (completionType == types_1.CompletionType.closingElement && parentTag)
                return _this.getClosingElementCompletion(parentTag);
            var namespaces = XsdNamespaces_1.XsdNamespaces.getXsdNamespaces(model);
            var xsdWorkers = XsdNamespaces_1.XsdNamespaces.getXsdWorkersForNamespace(namespaces, _this.xsdManager);
            var parentTagName = _this.getTagName(parentTag);
            var completions = [];
            xsdWorkers.forEach(function (xsdWorker) {
                completions = __spread(completions, xsdWorker.doCompletion(completionType, parentTagName ? parentTagName : parentTag));
            });
            return completions;
        };
        this.getCompletionType = function (model, position, context) {
            var wordsBeforePosition = _this.getWordsBeforePosition(model, position);
            var textUntilPosition = _this.getTextUntilPosition(model, position);
            if (_this.isInsideAttributeValue(wordsBeforePosition))
                return types_1.CompletionType.none;
            switch (context.triggerKind) {
                case CompletionTriggerKind.Invoke:
                    var completionType = _this.getCompletionTypeByPreviousText(textUntilPosition);
                    if (completionType)
                        return completionType;
                case CompletionTriggerKind.TriggerForIncompleteCompletions:
                    return _this.getCompletionTypeForIncompleteCompletion(wordsBeforePosition);
                case CompletionTriggerKind.TriggerCharacter:
                    return _this.getCompletionTypeByTriggerCharacter(context.triggerCharacter);
            }
        };
        this.isInsideAttributeValue = function (text) {
            var regexForInsideAttributeValue = /="[^"]+$/;
            var matches = text.match(regexForInsideAttributeValue);
            return !!matches;
        };
        this.getCompletionTypeForIncompleteCompletion = function (text) {
            var currentTag = _this.getTextFromCurrentTag(text);
            if (currentTag) {
                if (_this.textContainsAttributes(currentTag))
                    return types_1.CompletionType.incompleteAttribute;
                if (_this.textContainsTags(currentTag))
                    return types_1.CompletionType.incompleteElement;
            }
            return types_1.CompletionType.snippet;
        };
        this.getTextFromCurrentTag = function (text) {
            return SimpleParser_1.SimpleParser.getMatchesForRegex(text, /(<\/*[^>]*)$/g)[0];
        };
        this.textContainsAttributes = function (text) {
            return _this.getAttributesFromText(text).length > 0;
        };
        this.getAttributesFromText = function (text) {
            return SimpleParser_1.SimpleParser.getMatchesForRegex(text, /(?<=\s)[A-Za-z0-9_-]+/g);
        };
        this.textContainsTags = function (text) {
            var tags = _this.getTagsFromText(text);
            return tags !== undefined && tags.length > 0;
        };
        this.getTagsFromText = function (text) {
            return SimpleParser_1.SimpleParser.getMatchesForRegex(text, /(?<=<|<\/)[^?\s|/>]+(?!.*\/>)/g);
        };
        this.getCompletionTypeByTriggerCharacter = function (triggerCharacter) {
            switch (triggerCharacter) {
                case '<':
                    return types_1.CompletionType.element;
                case ' ':
                    return types_1.CompletionType.attribute;
                case '/':
                    return types_1.CompletionType.closingElement;
            }
            return types_1.CompletionType.none;
        };
        this.getCompletionTypeByPreviousText = function (text) {
            var lastCharacterBeforePosition = text[text.length - 1];
            switch (lastCharacterBeforePosition) {
                case '<':
                    return types_1.CompletionType.incompleteElement;
                case ' ':
                    return _this.getCompletionTypeAfterWhitespace(text);
                case '/':
                    return types_1.CompletionType.closingElement;
            }
        };
        this.getCompletionTypeAfterWhitespace = function (text) {
            if (_this.isInsideTag(text))
                return types_1.CompletionType.incompleteAttribute;
            if (_this.isAfterTag(text))
                return types_1.CompletionType.snippet;
        };
        this.isInsideTag = function (text) { return _this.getTextInsideCurrentTag(text).length > 0; };
        this.getTextInsideCurrentTag = function (text) {
            return SimpleParser_1.SimpleParser.getMatchesForRegex(text, /(?<=(<|<\/)[^?\s|/>]+)\s([\sA-Za-z0-9_\-="'])*$/g);
        };
        this.isAfterTag = function (text) { return _this.getTextAfterCurrentTag(text).length > 0; };
        this.getTextAfterCurrentTag = function (text) {
            return SimpleParser_1.SimpleParser.getMatchesForRegex(text, /(?<=>[\s\n]+)[^<]+$/g);
        };
        this.getParentTag = function (model, position) {
            var textUntilPosition = _this.getTextUntilPosition(model, position);
            var unclosedTags = _this.getUnclosedTags(textUntilPosition);
            var wordAtPosition = model.getWordAtPosition(position);
            if (_this.wordAtPositionIsEqualToLastUnclosedTag(wordAtPosition, unclosedTags))
                return unclosedTags[unclosedTags.length - 2];
            var lastTagBeforePosition = _this.getLastTagBeforePosition(model, position);
            var currentTagName = _this.getTagName(lastTagBeforePosition);
            if (wordAtPosition && currentTagName && currentTagName === wordAtPosition.word) {
                return unclosedTags[unclosedTags.length - 2];
            }
            return unclosedTags[unclosedTags.length - 1];
        };
        this.getTextUntilPosition = function (model, position) {
            return model.getValueInRange({
                startLineNumber: 1,
                startColumn: 1,
                endLineNumber: position.lineNumber,
                endColumn: position.column,
            });
        };
        this.getUnclosedTags = function (text) {
            var tags = _this.getTagsFromText(text);
            var parentTags = [];
            if (tags)
                tags.map(function (tag) {
                    if (parentTags.includes(tag)) {
                        while (parentTags[parentTags.length - 1] !== tag) {
                            parentTags.pop();
                        }
                        parentTags.pop();
                    }
                    else {
                        parentTags.push(tag);
                    }
                });
            return parentTags;
        };
        this.wordAtPositionIsEqualToLastUnclosedTag = function (wordAtPosition, unclosedTags) {
            return wordAtPosition !== null && wordAtPosition.word === unclosedTags[unclosedTags.length - 1];
        };
        this.getTagName = function (tag) {
            var currentTagParts = _this.getTagParts(tag);
            if (currentTagParts)
                return currentTagParts[1];
        };
        this.getTagParts = function (tag) {
            if (tag) {
                var tagParts = tag.split(':');
                if (tagParts.length > 1)
                    return tagParts;
            }
        };
        this.getLastTagBeforePosition = function (model, position) {
            var wordsBeforePosition = _this.getWordsBeforePosition(model, position);
            var tagsBeforePosition = _this.getTagsFromText(wordsBeforePosition);
            if (tagsBeforePosition && tagsBeforePosition.length > 0)
                return tagsBeforePosition[tagsBeforePosition.length - 1];
        };
        this.getWordsBeforePosition = function (model, position) {
            return model.getValueInRange({
                startLineNumber: position.lineNumber,
                startColumn: 0,
                endLineNumber: position.lineNumber,
                endColumn: position.column,
            });
        };
        this.getClosingElementCompletion = function (element) { return [
            {
                label: element,
                kind: CompletionItemKind.Property,
                detail: 'Close tag',
                insertText: element,
                documentation: "Closes the unclosed " + element + " tag in this file.",
            },
        ]; };
        this.xsdManager = xsdCollection;
    }
    return XsdCompletion;
}());
exports.default = XsdCompletion;
//# sourceMappingURL=XsdCompletion.js.map