"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var XsdCompletion_1 = __importDefault(require("./XsdCompletion"));
var XsdValidation_1 = __importDefault(require("./XsdValidation"));
var ts_debounce_1 = require("ts-debounce");
var XsdGenerateTemplate_1 = __importDefault(require("./XsdGenerateTemplate"));
var prettier_1 = __importDefault(require("prettier"));
var parser_html_1 = __importDefault(require("prettier/parser-html"));
var XsdFeatures = /** @class */ (function () {
    function XsdFeatures(xsdXollection, monaco, editor) {
        var _this = this;
        this.oldDecorations = [];
        this.addCompletion = function () {
            var xsdCompletion = new XsdCompletion_1.default(_this.xsdCollection);
            _this.monaco.languages.registerCompletionItemProvider('xml', xsdCompletion.provider());
        };
        this.doValidation = function () {
            var _a;
            console.log('validate');
            _this.xsdValidation = (_a = _this.xsdValidation) !== null && _a !== void 0 ? _a : new XsdValidation_1.default(_this.xsdCollection);
            var model = _this.editor.getModel();
            var newDecorations = _this.xsdValidation.decorations(model);
            _this.oldDecorations = _this.editor.deltaDecorations(_this.oldDecorations, newDecorations);
        };
        this.addValidation = function () {
            _this.xsdValidation = new XsdValidation_1.default(_this.xsdCollection);
            var debouncedDoValidation = ts_debounce_1.debounce(_this.doValidation, 1000);
            _this.editor.onKeyDown(debouncedDoValidation);
            _this.doValidation();
        };
        this.doReformatCode = function () {
            var model = _this.editor.getModel();
            if (model)
                model.setValue(_this.prettier(model.getValue()));
        };
        this.prettier = function (xml) {
            return prettier_1.default.format(xml, {
                parser: 'html',
                plugins: [parser_html_1.default],
                tabWidth: 4,
            });
        };
        this.doGenerate = function (levels, withAttributes) {
            var _a;
            _this.xsdGenerateTemplate = (_a = _this.xsdGenerateTemplate) !== null && _a !== void 0 ? _a : new XsdGenerateTemplate_1.default(_this.xsdCollection);
            var model = _this.editor.getModel();
            var template = _this.xsdGenerateTemplate.getTemplate(model, levels, withAttributes);
            if (template)
                model === null || model === void 0 ? void 0 : model.setValue(_this.prettier(template));
        };
        this.addGenerateAction = function () {
            _this.xsdGenerateTemplate = new XsdGenerateTemplate_1.default(_this.xsdCollection);
            _this.editor.addAction({
                id: 'xsd-template-generate-without-attributes',
                label: 'Generate Template from XSD',
                keybindings: [
                    _this.monaco.KeyMod.CtrlCmd | _this.monaco.KeyMod.Shift | _this.monaco.KeyCode.KEY_G,
                ],
                contextMenuGroupId: '1_modification',
                contextMenuOrder: 2,
                run: function () { var _a; return _this.doGenerate(parseInt((_a = window.prompt('levels', '1')) !== null && _a !== void 0 ? _a : ''), false); },
            });
            _this.editor.addAction({
                id: 'xsd-template-generate-with-attributes',
                label: 'Generate Template from XSD with requires attributes',
                keybindings: [
                    _this.monaco.KeyMod.CtrlCmd |
                        _this.monaco.KeyMod.Alt |
                        _this.monaco.KeyMod.Shift |
                        _this.monaco.KeyCode.KEY_G,
                ],
                contextMenuGroupId: '1_modification',
                contextMenuOrder: 3,
                run: function () { var _a; return _this.doGenerate(parseInt((_a = window.prompt('levels', '1')) !== null && _a !== void 0 ? _a : ''), true); },
            });
        };
        this.addReformatAction = function () {
            _this.editor.addAction({
                id: 'xsd-reformat-code',
                label: 'Reformat code',
                keybindings: [
                    _this.monaco.KeyMod.CtrlCmd | _this.monaco.KeyMod.Shift | _this.monaco.KeyCode.KEY_R,
                ],
                contextMenuGroupId: '1_modification',
                contextMenuOrder: 1.5,
                run: _this.doReformatCode,
            });
        };
        this.xsdCollection = xsdXollection;
        this.monaco = monaco;
        this.editor = editor;
        this.editor.updateOptions({
            wordSeparators: '`~!@#$%^&*()-=+[{]}\\|;\'",.<>/?',
        });
    }
    return XsdFeatures;
}());
exports.default = XsdFeatures;
//# sourceMappingURL=XsdFeatures.js.map