"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var monaco_editor_1 = require("monaco-editor");
var xmldom_1 = __importDefault(require("xmldom"));
var types_1 = require("./types");
var SimpleParser_1 = require("./SimpleParser");
var XsdValidation = /** @class */ (function () {
    function XsdValidation(xsdManager) {
        var _this = this;
        this.errors = [];
        this.model = null;
        this.decorations = function (model) {
            _this.errors = [];
            _this.model = model;
            var xml = model ? SimpleParser_1.SimpleParser.getFullText(model) : undefined;
            if (xml) {
                _this.dom.parseFromString(xml);
                // TODO: Implement XMLLint after XML DOM Parser
            }
            return _this.getDecorationsFromErrors();
        };
        this.getDecorationsFromErrors = function () {
            return _this.errors.map(_this.getDecorationsFromError);
        };
        this.getDecorationsFromError = function (error) { return ({
            range: _this.getWordRangeFromErrorPosition(error.position),
            options: {
                className: error.classNames,
                hoverMessage: [
                    {
                        value: error.errorTypeString,
                    },
                    {
                        value: error.errorMessage,
                    },
                    {
                        value: '*XML DOM Parser*',
                    },
                ],
            },
        }); };
        this.getWordRangeFromErrorPosition = function (position) {
            var _a, _b;
            var length = (_b = (_a = _this.model) === null || _a === void 0 ? void 0 : _a.getWordAtPosition(position)) === null || _b === void 0 ? void 0 : _b.word.length;
            if (length)
                return new monaco_editor_1.Range(position.lineNumber, position.column, position.lineNumber, position.column + length);
            else
                return new monaco_editor_1.Range(position.lineNumber, position.column - 1, position.lineNumber, position.column + 2);
        };
        this.xsdManager = xsdManager;
        this.dom = new xmldom_1.default.DOMParser({
            locator: {},
            errorHandler: {
                warning: function (message) {
                    return _this.errors.push(new types_1.XmlDomError(message, types_1.ErrorType.warning));
                },
                error: function (message) {
                    return _this.errors.push(new types_1.XmlDomError(message, types_1.ErrorType.error));
                },
                fatalError: function (message) {
                    return _this.errors.push(new types_1.XmlDomError(message, types_1.ErrorType.fetalError));
                },
            },
        });
    }
    return XsdValidation;
}());
exports.default = XsdValidation;
//# sourceMappingURL=XsdValidation.js.map